"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var js_ast_1 = require("./js-ast");
var crc32 = require("crc32");
var engine_1 = require("./engine");
var path = require("path");
var constants_1 = require("./constants");
var NodeKind;
(function (NodeKind) {
    NodeKind["Fragment"] = "Fragment";
    NodeKind["Text"] = "Text";
    NodeKind["Element"] = "Element";
    NodeKind["StyleElement"] = "StyleElement";
    NodeKind["Slot"] = "Slot";
    NodeKind["Block"] = "Block";
})(NodeKind = exports.NodeKind || (exports.NodeKind = {}));
var AttributeKind;
(function (AttributeKind) {
    AttributeKind["ShorthandAttribute"] = "ShorthandAttribute";
    AttributeKind["KeyValueAttribute"] = "KeyValueAttribute";
    AttributeKind["SpreadAttribute"] = "SpreadAttribute";
})(AttributeKind = exports.AttributeKind || (exports.AttributeKind = {}));
var AttributeValueKind;
(function (AttributeValueKind) {
    AttributeValueKind["String"] = "String";
    AttributeValueKind["Slot"] = "Slot";
})(AttributeValueKind = exports.AttributeValueKind || (exports.AttributeValueKind = {}));
var BlockKind;
(function (BlockKind) {
    BlockKind["Each"] = "Each";
    BlockKind["Conditional"] = "Conditional";
})(BlockKind = exports.BlockKind || (exports.BlockKind = {}));
var ConditionalBlockKind;
(function (ConditionalBlockKind) {
    ConditionalBlockKind["PassFailBlock"] = "PassFailBlock";
    ConditionalBlockKind["FinalBlock"] = "FinalBlock";
})(ConditionalBlockKind = exports.ConditionalBlockKind || (exports.ConditionalBlockKind = {}));
exports.getImports = function (ast) {
    return exports.getChildrenByTagName("import", ast).filter(function (child) {
        return exports.hasAttribute("src", child);
    });
};
exports.getRelativeFilePath = function (fromFilePath, importFilePath) {
    var logicPath = engine_1.resolveImportFile(fromFilePath, importFilePath);
    var relativePath = path.relative(path.dirname(fromFilePath), logicPath);
    if (relativePath.charAt(0) !== ".") {
        relativePath = "./" + relativePath;
    }
    return relativePath;
};
exports.getImportIds = function (ast) {
    return exports.getImports(ast)
        .map(function (node) { return exports.getAttributeStringValue("id", node); })
        .filter(Boolean);
};
exports.getChildren = function (ast) {
    if (ast.kind === NodeKind.Element || ast.kind === NodeKind.Fragment) {
        return ast.children;
    }
    return [];
};
exports.getStyleScopes = function (ast, filePath) {
    var scopes = [];
    if (exports.getStyleElements(ast).length > 0) {
        scopes.push(crc32("file://" + filePath));
    }
    for (var _i = 0, _a = exports.getImports(ast); _i < _a.length; _i++) {
        var imp = _a[_i];
        var src = exports.getAttributeStringValue("src", imp);
        if (/\.css$/.test(src)) {
            var cssFilePath = engine_1.resolveImportFile(filePath, src);
            scopes.push(crc32("file://" + cssFilePath));
        }
    }
    return scopes;
};
exports.getChildrenByTagName = function (tagName, parent) {
    return exports.getChildren(parent).filter(function (child) {
        return (child.kind === NodeKind.Element &&
            child.tagName === tagName &&
            exports.hasAttribute("src", child));
    });
};
exports.getMetaValue = function (name, root) {
    var metaElement = exports.getChildrenByTagName("meta", root).find(function (meta) { return exports.getAttributeStringValue("name", meta) === name; });
    return metaElement && exports.getAttributeStringValue("content", metaElement);
};
exports.getAttribute = function (name, element) {
    return element.attributes.find(function (attr) {
        return attr.kind === AttributeKind.KeyValueAttribute && attr.name === name;
    });
};
exports.getAttributeValue = function (name, element) {
    var attr = exports.getAttribute(name, element);
    return attr && attr.value;
};
exports.getAttributeStringValue = function (name, element) {
    var value = exports.getAttributeValue(name, element);
    return (value && value.attrValueKind === AttributeValueKind.String && value.value);
};
exports.getStyleElements = function (ast) {
    return exports.getChildren(ast).filter(function (child) { return child.kind === NodeKind.StyleElement; });
};
exports.isVisibleElement = function (ast) {
    return !/^(import|logic|meta|style|part|preview)$/.test(ast.tagName);
};
exports.isVisibleNode = function (node) {
    return node.kind === NodeKind.Text ||
        node.kind === NodeKind.Fragment ||
        node.kind === NodeKind.Slot ||
        node.kind === NodeKind.Block ||
        (node.kind === NodeKind.Element && exports.isVisibleElement(node));
};
exports.getVisibleChildNodes = function (ast) {
    return exports.getChildren(ast).filter(exports.isVisibleNode);
};
exports.getParts = function (ast) {
    return exports.getChildren(ast).filter(function (child) {
        return (child.kind === NodeKind.Element &&
            child.tagName === constants_1.PART_TAG_NAME &&
            exports.hasAttribute("id", child));
    });
};
exports.getPartIds = function (ast) {
    return exports.getParts(ast)
        .map(function (node) { return exports.getAttributeStringValue("id", node); })
        .filter(Boolean);
};
exports.getDefaultPart = function (ast) {
    return exports.getParts(ast).find(function (part) { return exports.getAttributeStringValue("id", part) === constants_1.DEFAULT_PART_ID; });
};
exports.getLogicElement = function (ast) {
    return exports.getChildren(ast).find(function (child) { return child.kind === NodeKind.Element && child.tagName === constants_1.LOGIC_TAG_NAME; });
};
exports.hasAttribute = function (name, element) {
    return exports.getAttribute(name, element) != null;
};
exports.flattenNodes = function (node, _allNodes) {
    if (_allNodes === void 0) { _allNodes = []; }
    _allNodes.push(node);
    if (node.kind === NodeKind.Element) {
        for (var _i = 0, _a = node.attributes; _i < _a.length; _i++) {
            var attr = _a[_i];
            if (attr.kind === AttributeKind.KeyValueAttribute && attr.value) {
                if (attr.value.attrValueKind === AttributeValueKind.Slot) {
                    if (attr.value.jsKind === js_ast_1.StatementKind.Node) {
                        exports.flattenNodes(attr.value, _allNodes);
                    }
                }
            }
        }
    }
    if (node.kind === NodeKind.Block) {
        if (node.blockKind === BlockKind.Each) {
            if (node.body) {
                exports.flattenNodes(node.body, _allNodes);
            }
        }
        else if (node.blockKind === BlockKind.Conditional) {
            flattenConditional(node, _allNodes);
        }
    }
    for (var _b = 0, _c = exports.getChildren(node); _b < _c.length; _b++) {
        var child = _c[_b];
        exports.flattenNodes(child, _allNodes);
    }
    return _allNodes;
};
var flattenConditional = function (conditional, _allNodes) {
    if (conditional.body) {
        exports.flattenNodes(conditional.body, _allNodes);
    }
    if (conditional.conditionalBlockKind === ConditionalBlockKind.PassFailBlock) {
        if (conditional.fail) {
            flattenConditional(conditional.fail, _allNodes);
        }
    }
    return _allNodes;
};
exports.isComponentInstance = function (node, importIds) {
    return (node.kind === NodeKind.Element &&
        importIds.indexOf(node.tagName.split(".").shift()) !== -1);
};
var maybeAddReference = function (stmt, _statements) {
    if (_statements === void 0) { _statements = []; }
    if (stmt.jsKind === js_ast_1.StatementKind.Reference) {
        _statements.push([stmt, null]);
    }
};
exports.getNestedReferences = function (node, _statements) {
    if (_statements === void 0) { _statements = []; }
    if (node.kind === NodeKind.Slot) {
        maybeAddReference(node.script, _statements);
    }
    else if (node.kind === NodeKind.Block) {
        if (node.blockKind === BlockKind.Each) {
            // if (node.body) {
            //   getNestedReferences(node.body, _statements);
            // }
            maybeAddReference(node.source, _statements);
        }
        else if (node.blockKind === BlockKind.Conditional) {
            var current = node;
            while (current) {
                if (current.body) {
                    exports.getNestedReferences(current.body, _statements);
                }
                if (current.conditionalBlockKind === ConditionalBlockKind.PassFailBlock) {
                    maybeAddReference(current.condition, _statements);
                    current = current.fail;
                }
                else {
                    // final block
                    break;
                }
            }
        }
    }
    else {
        if (node.kind === NodeKind.Element) {
            for (var _i = 0, _a = node.attributes; _i < _a.length; _i++) {
                var attr = _a[_i];
                if (attr.kind == AttributeKind.KeyValueAttribute &&
                    attr.value &&
                    attr.value.attrValueKind === AttributeValueKind.Slot) {
                    if (attr.value.jsKind === js_ast_1.StatementKind.Node) {
                        exports.getNestedReferences(attr.value, _statements);
                    }
                    else if (attr.value.jsKind === js_ast_1.StatementKind.Reference) {
                        _statements.push([attr.value, attr.name]);
                    }
                }
                else if (attr.kind === AttributeKind.ShorthandAttribute &&
                    attr.reference.jsKind === js_ast_1.StatementKind.Reference) {
                    _statements.push([attr.reference, attr.reference[0]]);
                }
                else if (attr.kind === AttributeKind.SpreadAttribute &&
                    attr.script.jsKind === js_ast_1.StatementKind.Reference) {
                    _statements.push([attr.script, attr.script[0]]);
                }
            }
        }
        for (var _b = 0, _c = exports.getChildren(node); _b < _c.length; _b++) {
            var child = _c[_b];
            if (child.kind === NodeKind.Element &&
                child.tagName === constants_1.PREVIEW_TAG_NAME) {
                continue;
            }
            exports.getNestedReferences(child, _statements);
        }
    }
    return _statements;
};
