"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.evaluateAllFileStyles = exports.Engine = void 0;
var fs = require("fs");
var url = require("url");
var paperclip_1 = require("../native/pkg/paperclip");
var paperclip_utils_1 = require("paperclip-utils");
var mapResult = function (result) {
    if (!result) {
        return result;
    }
    if (result.Ok) {
        return result.Ok;
    }
    else {
        return { error: result.Err };
    }
};
var Engine = /** @class */ (function () {
    function Engine(_options, _onCrash) {
        var _this = this;
        if (_options === void 0) { _options = {}; }
        if (_onCrash === void 0) { _onCrash = function () { }; }
        this._options = _options;
        this._onCrash = _onCrash;
        this._listeners = [];
        this._tryCatch = function (fn) {
            try {
                return fn();
            }
            catch (e) {
                _this._onCrash();
                return null;
            }
        };
        this._dispatch = function (event) {
            // try-catch since engine will throw opaque error.
            try {
                for (var _i = 0, _a = _this._listeners; _i < _a.length; _i++) {
                    var listener = _a[_i];
                    listener(event);
                }
            }
            catch (e) {
                console.error(e);
                throw e;
            }
        };
        var io = Object.assign({
            readFile: function (uri) {
                return fs.readFileSync(new URL(uri), "utf8");
            },
            fileExists: function (uri) {
                try {
                    var url_1 = new URL(uri);
                    return fs.existsSync(url_1) && fs.lstatSync(url_1).isFile();
                }
                catch (e) {
                    console.error(e);
                    return false;
                }
            },
            resolveFile: paperclip_utils_1.resolveImportUri(fs)
        }, _options.io);
        var initNative = function () {
            _this._native = paperclip_1.NativeEngine.new(io.readFile, io.fileExists, io.resolveFile);
            // only one native listener to for buffer performance
            _this._native.add_listener(_this._dispatch);
        };
        initNative();
    }
    Engine.prototype.onEvent = function (listener) {
        var _this = this;
        if (listener == null) {
            throw new Error("listener cannot be undefined");
        }
        this._listeners.push(listener);
        return function () {
            var i = _this._listeners.indexOf(listener);
            if (i !== -1) {
                _this._listeners.splice(i, 1);
            }
        };
    };
    Engine.prototype.parseFile = function (uri) {
        return mapResult(this._native.parse_file(uri));
    };
    Engine.prototype.getLoadedAst = function (uri) {
        var _this = this;
        return this._tryCatch(function () { return _this._native.get_loaded_ast(uri); });
    };
    Engine.prototype.evaluateFileStyles = function (uri) {
        var _this = this;
        return this._tryCatch(function () {
            return mapResult(_this._native.evaluate_file_styles(uri));
        });
    };
    Engine.prototype.evaluateContentStyles = function (content, uri) {
        var _this = this;
        return this._tryCatch(function () {
            return mapResult(_this._native.evaluate_content_styles(content, uri));
        });
    };
    Engine.prototype.parseContent = function (content) {
        var _this = this;
        return this._tryCatch(function () { return mapResult(_this._native.parse_content(content)); });
    };
    Engine.prototype.updateVirtualFileContent = function (uri, content) {
        var _this = this;
        this._dispatch({ kind: paperclip_utils_1.EngineEventKind.Updating, uri: uri });
        return this._tryCatch(function () {
            return mapResult(_this._native.update_virtual_file_content(uri, content));
        });
    };
    Engine.prototype.load = function (uri) {
        var _this = this;
        this._dispatch({ kind: paperclip_utils_1.EngineEventKind.Loading, uri: uri });
        return this._tryCatch(function () {
            return mapResult(_this._native.load(uri, _this._options.renderPart));
        });
    };
    return Engine;
}());
exports.Engine = Engine;
exports.evaluateAllFileStyles = function (engine, ast, resourceUrl, _loadedStyleFiles) {
    if (_loadedStyleFiles === void 0) { _loadedStyleFiles = {}; }
    var imports = paperclip_utils_1.getImports(ast);
    var map = {};
    for (var _i = 0, imports_1 = imports; _i < imports_1.length; _i++) {
        var imp = imports_1[_i];
        var src = paperclip_utils_1.getAttributeStringValue("src", imp);
        if (/\.css$/.test(src)) {
            var cssFilePath = paperclip_utils_1.resolveImportFile(fs)(resourceUrl, src);
            var transformPath = url.fileURLToPath(cssFilePath);
            var importedSheetCode = _loadedStyleFiles[cssFilePath];
            if (!importedSheetCode) {
                importedSheetCode = engine.evaluateFileStyles(cssFilePath);
            }
            map[transformPath] = importedSheetCode;
        }
    }
    return map;
};
